@@ -1,39 +1,45 @@
 /* liblxcapi
  *
- * Copyright © 2017 Christian Brauner <christian.brauner@ubuntu.com>.
- * Copyright © 2017 Canonical Ltd.
+ * Copyright © 2019 Christian Brauner <christian.brauner@ubuntu.com>.
+ * Copyright © 2019 Canonical Ltd.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
 #define __STDC_FORMAT_MACROS /* Required for PRIu64 to work. */
 #include <errno.h>
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <unistd.h>
 
 #include "af_unix.h"
 #include "commands.h"
 #include "commands_utils.h"
+#include "config.h"
+#include "file_utils.h"
 #include "initutils.h"
 #include "log.h"
 #include "lxclock.h"
+#include "memory_utils.h"
 #include "monitor.h"
 #include "state.h"
 #include "utils.h"
@@ -52,7 +58,7 @@ int lxc_cmd_sock_rcv_state(int state_client_fd, int timeout)
 		ret = setsockopt(state_client_fd, SOL_SOCKET, SO_RCVTIMEO,
 				(const void *)&out, sizeof(out));
 		if (ret < 0) {
-			SYSERROR("Failed to set %ds timeout on containter "
+			SYSERROR("Failed to set %ds timeout on container "
 				 "state socket",
 				 timeout);
 			return -1;
@@ -61,14 +67,8 @@ int lxc_cmd_sock_rcv_state(int state_client_fd, int timeout)
 
 	memset(&msg, 0, sizeof(msg));
 
-again:
-	ret = recv(state_client_fd, &msg, sizeof(msg), 0);
+	ret = lxc_recv_nointr(state_client_fd, &msg, sizeof(msg), 0);
 	if (ret < 0) {
-		if (errno == EINTR) {
-			TRACE("Caught EINTR; retrying");
-			goto again;
-		}
-
 		SYSERROR("Failed to receive message");
 		return -1;
 	}
@@ -83,8 +83,8 @@ again:
 int lxc_cmd_sock_get_state(const char *name, const char *lxcpath,
 			   lxc_state_t states[MAX_STATE], int timeout)
 {
+	__do_close_prot_errno int state_client_fd = -EBADF;
 	int ret;
-	int state_client_fd;
 
 	ret = lxc_cmd_add_state_client(name, lxcpath, states, &state_client_fd);
 	if (ret < 0)
@@ -93,29 +93,41 @@ int lxc_cmd_sock_get_state(const char *name, const char *lxcpath,
 	if (ret < MAX_STATE)
 		return ret;
 
-	ret = lxc_cmd_sock_rcv_state(state_client_fd, timeout);
-	close(state_client_fd);
-	return ret;
+	return lxc_cmd_sock_rcv_state(state_client_fd, timeout);
 }
 
-int lxc_make_abstract_socket_name(char *path, int len, const char *lxcname,
+int lxc_make_abstract_socket_name(char *path, size_t pathlen,
+				  const char *lxcname,
 				  const char *lxcpath,
 				  const char *hashed_sock_name,
 				  const char *suffix)
 {
+	__do_free char *tmppath = NULL;
 	const char *name;
-	char *tmppath;
+	char *offset;
+	size_t len;
 	size_t tmplen;
 	uint64_t hash;
 	int ret;
 
+	if (!path)
+		return -1;
+
+	offset = &path[1];
+
+	/* -2 here because this is an abstract unix socket so it needs a
+	 * leading \0, and we null terminate, so it needs a trailing \0.
+	 * Although null termination isn't required by the API, we do it anyway
+	 * because we print the sockname out sometimes.
+	 */
+	len = pathlen - 2;
+
 	name = lxcname;
 	if (!name)
 		name = "";
 
 	if (hashed_sock_name != NULL) {
-		ret =
-		    snprintf(path, len, "lxc/%s/%s", hashed_sock_name, suffix);
+		ret = snprintf(offset, len, "lxc/%s/%s", hashed_sock_name, suffix);
 		if (ret < 0 || ret >= len) {
 			ERROR("Failed to create abstract socket name");
 			return -1;
@@ -131,7 +143,7 @@ int lxc_make_abstract_socket_name(char *path, int len, const char *lxcname,
 		}
 	}
 
-	ret = snprintf(path, len, "%s/%s/%s", lxcpath, name, suffix);
+	ret = snprintf(offset, len, "%s/%s/%s", lxcpath, name, suffix);
 	if (ret < 0) {
 		ERROR("Failed to create abstract socket name");
 		return -1;
@@ -141,7 +153,7 @@ int lxc_make_abstract_socket_name(char *path, int len, const char *lxcname,
 
 	/* ret >= len; lxcpath or name is too long.  hash both */
 	tmplen = strlen(name) + strlen(lxcpath) + 2;
-	tmppath = alloca(tmplen);
+	tmppath = must_realloc(NULL, tmplen);
 	ret = snprintf(tmppath, tmplen, "%s/%s", lxcpath, name);
 	if (ret < 0 || (size_t)ret >= tmplen) {
 		ERROR("Failed to create abstract socket name");
@@ -149,7 +161,7 @@ int lxc_make_abstract_socket_name(char *path, int len, const char *lxcname,
 	}
 
 	hash = fnv_64a_buf(tmppath, ret, FNV1A_64_INIT);
-	ret = snprintf(path, len, "lxc/%016" PRIx64 "/%s", hash, suffix);
+	ret = snprintf(offset, len, "lxc/%016" PRIx64 "/%s", hash, suffix);
 	if (ret < 0 || ret >= len) {
 		ERROR("Failed to create abstract socket name");
 		return -1;
@@ -162,27 +174,17 @@ int lxc_cmd_connect(const char *name, const char *lxcpath,
 		    const char *hashed_sock_name, const char *suffix)
 {
 	int ret, client_fd;
-	char path[sizeof(((struct sockaddr_un *)0)->sun_path)] = {0};
-	char *offset = &path[1];
+	char path[LXC_AUDS_ADDR_LEN] = {0};
 
-	/* -2 here because this is an abstract unix socket so it needs a
-	 * leading \0, and we null terminate, so it needs a trailing \0.
-	 * Although null termination isn't required by the API, we do it anyway
-	 * because we print the sockname out sometimes.
-	 */
-	size_t len = sizeof(path) - 2;
-	ret = lxc_make_abstract_socket_name(offset, len, name, lxcpath,
+	ret = lxc_make_abstract_socket_name(path, sizeof(path), name, lxcpath,
 					    hashed_sock_name, suffix);
 	if (ret < 0)
 		return -1;
 
 	/* Get new client fd. */
 	client_fd = lxc_abstract_unix_connect(path);
-	if (client_fd < 0) {
-		if (errno == ECONNREFUSED)
-			return -ECONNREFUSED;
+	if (client_fd < 0)
 		return -1;
-	}
 
 	return client_fd;
 }
@@ -190,9 +192,9 @@ int lxc_cmd_connect(const char *name, const char *lxcpath,
 int lxc_add_state_client(int state_client_fd, struct lxc_handler *handler,
 			 lxc_state_t states[MAX_STATE])
 {
+	__do_free struct lxc_state_client *newclient = NULL;
+	__do_free struct lxc_list *tmplist = NULL;
 	int state;
-	struct lxc_state_client *newclient;
-	struct lxc_list *tmplist;
 
 	newclient = malloc(sizeof(*newclient));
 	if (!newclient)
@@ -203,21 +205,19 @@ int lxc_add_state_client(int state_client_fd, struct lxc_handler *handler,
 	newclient->clientfd = state_client_fd;
 
 	tmplist = malloc(sizeof(*tmplist));
-	if (!tmplist) {
-		free(newclient);
+	if (!tmplist)
 		return -ENOMEM;
-	}
 
 	state = handler->state;
 	if (states[state] != 1) {
 		lxc_list_add_elem(tmplist, newclient);
 		lxc_list_add_tail(&handler->conf->state_clients, tmplist);
 	} else {
-		free(newclient);
-		free(tmplist);
 		return state;
 	}
 
-	TRACE("added state client %d to state client list", state_client_fd);
+	TRACE("Added state client %d to state client list", state_client_fd);
+	move_ptr(newclient);
+	move_ptr(tmplist);
 	return MAX_STATE;
 }
