@@ -20,20 +20,25 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#include <sys/socket.h>
-#include <string.h>
-#include <stdio.h>
-#include <time.h>
-#include <unistd.h>
+
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
 #include <errno.h>
-#include <stdlib.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <time.h>
+#include <unistd.h>
 
+#include "config.h"
+#include "log.h"
 #include "nl.h"
 
-#define NLMSG_TAIL(nmsg) \
-        ((struct rtattr *) (((void *) (nmsg)) + NLMSG_ALIGN((nmsg)->nlmsg_len)))
+lxc_log_define(nl, lxc);
 
 extern size_t nlmsg_len(const struct nlmsg *nlmsg)
 {
@@ -204,8 +209,10 @@ again:
 	if (!ret)
 		return 0;
 
-	if (msg.msg_flags & MSG_TRUNC && (ret == nlmsghdr->nlmsg_len))
-		return -EMSGSIZE;
+	if (msg.msg_flags & MSG_TRUNC && (ret == nlmsghdr->nlmsg_len)) {
+		errno = EMSGSIZE;
+		ret = -1;
+	}
 
 	return ret;
 }
@@ -255,18 +262,21 @@ extern int __netlink_transaction(struct nl_handler *handler,
 
 	ret = __netlink_send(handler, request);
 	if (ret < 0)
-		return ret;
+		return -1;
 
 	ret = __netlink_recv(handler, answer);
 	if (ret < 0)
-		return ret;
+		return -1;
 
+	ret = 0;
 	if (answer->nlmsg_type == NLMSG_ERROR) {
 		struct nlmsgerr *err = (struct nlmsgerr *)NLMSG_DATA(answer);
-		return err->error;
+		errno = -err->error;
+		if (err->error < 0)
+			ret = -1;
 	}
 
-	return 0;
+	return ret;
 }
 
 extern int netlink_transaction(struct nl_handler *handler,
@@ -285,7 +295,7 @@ extern int netlink_open(struct nl_handler *handler, int protocol)
 
 	memset(handler, 0, sizeof(*handler));
 
-	handler->fd = socket(AF_NETLINK, SOCK_RAW, protocol);
+	handler->fd = socket(AF_NETLINK, SOCK_RAW | SOCK_CLOEXEC, protocol);
 	if (handler->fd < 0)
 		return -errno;
 
@@ -337,3 +347,22 @@ extern int netlink_close(struct nl_handler *handler)
 	return 0;
 }
 
+int addattr(struct nlmsghdr *n, size_t maxlen, int type, const void *data,
+	    size_t alen)
+{
+	int len = RTA_LENGTH(alen);
+	struct rtattr *rta;
+
+	errno = EMSGSIZE;
+	if (NLMSG_ALIGN(n->nlmsg_len) + RTA_ALIGN(len) > maxlen)
+		return -1;
+
+	rta = NLMSG_TAIL(n);
+	rta->rta_type = type;
+	rta->rta_len = len;
+	if (alen)
+		memcpy(RTA_DATA(rta), data, alen);
+	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + RTA_ALIGN(len);
+
+	return 0;
+}
