@@ -102,6 +102,18 @@ struct lxc_handler {
 	/* The child's pid. */
 	pid_t pid;
 
+	/* The child's pidfd. */
+	int pidfd;
+
+	/*
+	 * File descriptor for the /proc/<pid> directory of the container's
+	 * init process.
+	 */
+	int proc_pidfd;
+
+	/* The monitor's pid. */
+	pid_t monitor_pid;
+
 	/* Whether the child has already exited. */
 	bool init_died;
 
