@@ -41,6 +41,7 @@
 #include "btrfs.h"
 #include "config.h"
 #include "log.h"
+#include "memory_utils.h"
 #include "rsync.h"
 #include "storage.h"
 #include "utils.h"
@@ -763,8 +764,6 @@ static int btrfs_recursive_destroy(const char *path)
 	int ret, e, i;
 	unsigned long off = 0;
 	u16 name_len;
-	char *name;
-	char *tmppath;
 	u64 dir_id;
 
 	fd = open(path, O_RDONLY);
@@ -805,7 +804,7 @@ static int btrfs_recursive_destroy(const char *path)
 	sk->max_transid = (u64)-1;
 	sk->nr_items = 4096;
 
-	while(1) {
+	for (;;) {
 		ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
 		e = errno;
 		if (ret < 0) {
@@ -835,9 +834,22 @@ static int btrfs_recursive_destroy(const char *path)
 			 * name of the child subvol in question.
 			 */
 			if (sh.objectid != root_id && sh.type == BTRFS_ROOT_BACKREF_KEY) {
+				__do_free char *name = NULL, *tmppath = NULL;
+				char *tmp;
+
 				ref = (struct btrfs_root_ref *)(args.buf + off);
 				name_len = btrfs_stack_root_ref_name_len(ref);
-				name = (char *)(ref + 1);
+				tmp = (char *)(ref + 1);
+
+				name = malloc(name_len + 1);
+				if (!name) {
+					ERROR("Out of memory");
+					free_btrfs_tree(tree);
+					close(fd);
+				}
+
+				memcpy(name, tmp, name_len);
+				name[name_len] = '\0';
 				dir_id = btrfs_stack_root_ref_dirid(ref);
 				tmppath = get_btrfs_subvol_path(fd, sh.offset,
 						dir_id, name, name_len);
@@ -847,12 +859,9 @@ static int btrfs_recursive_destroy(const char *path)
 							name_len, tmppath)) {
 					ERROR("Out of memory");
 					free_btrfs_tree(tree);
-					free(tmppath);
 					close(fd);
 					return -1;
 				}
-
-				free(tmppath);
 			}
 
 			off += sh.len;
