@@ -21,20 +21,23 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <sys/sysmacros.h>
 #include <sys/wait.h>
+#include <unistd.h>
 
 #include "config.h"
 #include "log.h"
 #include "lvm.h"
+#include "memory_utils.h"
 #include "rsync.h"
 #include "storage.h"
 #include "storage_utils.h"
@@ -52,6 +55,7 @@ struct lvcreate_args {
 	const char *lv;
 	const char *thinpool;
 	const char *fstype;
+	bool sigwipe;
 
 	/* snapshot specific arguments */
 	const char *source_lv;
@@ -73,12 +77,19 @@ static int lvm_create_exec_wrapper(void *data)
 
 	(void)setenv("LVM_SUPPRESS_FD_WARNINGS", "1", 1);
 	if (args->thinpool)
-		execlp("lvcreate", "lvcreate", "--thinpool", args->thinpool,
-		       "-V", args->size, args->vg, "-n", args->lv,
-		       (char *)NULL);
+		if(args->sigwipe)
+			execlp("lvcreate", "lvcreate", "-Wy", "--yes", "--thinpool", args->thinpool,
+			       "-V", args->size, args->vg, "-n", args->lv, (char *)NULL);
+		else
+			execlp("lvcreate", "lvcreate", "-qq", "--thinpool", args->thinpool,
+			       "-V", args->size, args->vg, "-n", args->lv, (char *)NULL);
 	else
-		execlp("lvcreate", "lvcreate", "-L", args->size, args->vg, "-n",
-		       args->lv, (char *)NULL);
+		if(args->sigwipe)
+			execlp("lvcreate", "lvcreate", "-Wy", "--yes", "-L", args->size, args->vg, "-n",
+			       args->lv, (char *)NULL);
+		else
+			execlp("lvcreate", "lvcreate", "-qq", "-L", args->size, args->vg, "-n",
+			       args->lv, (char *)NULL);
 
 	return -1;
 }
@@ -109,9 +120,9 @@ static int do_lvm_create(const char *path, uint64_t size, const char *thinpool)
 {
 	int len, ret;
 	char *pathdup, *vg, *lv;
-	char cmd_output[MAXPATHLEN];
+	char cmd_output[PATH_MAX];
 	char sz[24];
-	char *tp = NULL;
+	__do_free char *tp = NULL;
 	struct lvcreate_args cmd_args = {0};
 
 	ret = snprintf(sz, 24, "%" PRIu64 "b", size);
@@ -147,7 +158,7 @@ static int do_lvm_create(const char *path, uint64_t size, const char *thinpool)
 
 	if (thinpool) {
 		len = strlen(pathdup) + strlen(thinpool) + 2;
-		tp = alloca(len);
+		tp = must_realloc(NULL, len);
 
 		ret = snprintf(tp, len, "%s/%s", pathdup, thinpool);
 		if (ret < 0 || ret >= len) {
@@ -174,11 +185,22 @@ static int do_lvm_create(const char *path, uint64_t size, const char *thinpool)
 	cmd_args.vg = vg;
 	cmd_args.lv = lv;
 	cmd_args.size = sz;
+	cmd_args.sigwipe = true;
 	TRACE("Creating new lvm storage volume \"%s\" on volume group \"%s\" "
 	      "of size \"%s\"", lv, vg, sz);
-	ret = run_command(cmd_output, sizeof(cmd_output),
+	ret = run_command_status(cmd_output, sizeof(cmd_output),
 			  lvm_create_exec_wrapper, (void *)&cmd_args);
-	if (ret < 0) {
+
+	/* If lvcreate is old and doesn't support signature wiping, try again without it.
+	 * Test for exit code EINVALID_CMD_LINE(3) of lvcreate command.
+	 */
+	if (WIFEXITED(ret) && WEXITSTATUS(ret) == 3) {
+		cmd_args.sigwipe = false;
+		ret = run_command(cmd_output, sizeof(cmd_output),
+			      lvm_create_exec_wrapper, (void *)&cmd_args);
+	}
+
+	if (ret != 0) {
 		ERROR("Failed to create logical volume \"%s\": %s", lv,
 		      cmd_output);
 		free(pathdup);
@@ -199,7 +221,7 @@ bool lvm_detect(const char *path)
 	int fd;
 	ssize_t ret;
 	struct stat statbuf;
-	char devp[MAXPATHLEN], buf[4];
+	char devp[PATH_MAX], buf[4];
 
 	if (!strncmp(path, "lvm:", 4))
 		return true;
@@ -211,9 +233,9 @@ bool lvm_detect(const char *path)
 	if (!S_ISBLK(statbuf.st_mode))
 		return false;
 
-	ret = snprintf(devp, MAXPATHLEN, "/sys/dev/block/%d:%d/dm/uuid",
+	ret = snprintf(devp, PATH_MAX, "/sys/dev/block/%d:%d/dm/uuid",
 		       major(statbuf.st_rdev), minor(statbuf.st_rdev));
-	if (ret < 0 || ret >= MAXPATHLEN) {
+	if (ret < 0 || ret >= PATH_MAX) {
 		ERROR("Failed to create string");
 		return false;
 	}
@@ -262,20 +284,20 @@ int lvm_umount(struct lxc_storage *bdev)
 	return umount(bdev->dest);
 }
 
+#define __LVSCMD "lvs --unbuffered --noheadings -o lv_attr %s 2>/dev/null"
 int lvm_compare_lv_attr(const char *path, int pos, const char expected)
 {
+	__do_free char *cmd = NULL;
 	struct lxc_popen_FILE *f;
 	int ret, status;
 	size_t len;
-	char *cmd;
 	char output[12];
 	int start = 0;
-	const char *lvscmd = "lvs --unbuffered --noheadings -o lv_attr %s 2>/dev/null";
 
-	len = strlen(lvscmd) + strlen(path) + 1;
-	cmd = alloca(len);
+	len = strlen(__LVSCMD) + strlen(path) + 1;
+	cmd = must_realloc(NULL, len);
 
-	ret = snprintf(cmd, len, lvscmd, path);
+	ret = snprintf(cmd, len, __LVSCMD, path);
 	if (ret < 0 || (size_t)ret >= len)
 		return -1;
 
@@ -333,7 +355,7 @@ static int lvm_snapshot(struct lxc_storage *orig, const char *path, uint64_t siz
 	char *lv, *pathdup;
 	char sz[24];
 	char fstype[100];
-	char cmd_output[MAXPATHLEN];
+	char cmd_output[PATH_MAX];
 	char repairchar;
 	const char *origsrc;
 	struct lvcreate_args cmd_args = {0};
@@ -504,7 +526,7 @@ bool lvm_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
 	const char *thinpool;
 	struct rsync_data data;
 	const char *cmd_args[2];
-	char cmd_output[MAXPATHLEN] = {0};
+	char cmd_output[PATH_MAX] = {0};
 	char fstype[100] = "ext4";
 	uint64_t size = newsize;
 
@@ -513,13 +535,13 @@ bool lvm_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
 		if (!newsize && blk_getsize(orig, &size) < 0) {
 			ERROR("Failed to detect size of logical volume \"%s\"",
 			      orig->src);
-			return -1;
+			return false;
 		}
 
 		/* detect filesystem */
 		if (detect_fs(orig, fstype, 100) < 0) {
 			INFO("Failed to detect filesystem type for \"%s\"", orig->src);
-			return -1;
+			return false;
 		}
 	} else if (!newsize) {
 			size = DEFAULT_FS_SIZE;
@@ -531,7 +553,7 @@ bool lvm_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
 	ret = do_lvm_create(src, size, thinpool);
 	if (ret < 0) {
 		ERROR("Failed to create lvm storage volume \"%s\"", src);
-		return -1;
+		return false;
 	}
 
 	cmd_args[0] = fstype;
@@ -541,7 +563,7 @@ bool lvm_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
 	if (ret < 0) {
 		ERROR("Failed to create new filesystem \"%s\" for lvm storage "
 		      "volume \"%s\": %s", fstype, src, cmd_output);
-		return -1;
+		return false;
 	}
 
 	data.orig = orig;
@@ -591,7 +613,7 @@ bool lvm_create_snapshot(struct lxc_conf *conf, struct lxc_storage *orig,
 int lvm_destroy(struct lxc_storage *orig)
 {
 	int ret;
-	char cmd_output[MAXPATHLEN];
+	char cmd_output[PATH_MAX];
 	struct lvcreate_args cmd_args = {0};
 
 	cmd_args.lv = lxc_storage_get_path(orig->src, "lvm");
@@ -614,7 +636,7 @@ int lvm_create(struct lxc_storage *bdev, const char *dest, const char *n,
 	uint64_t sz;
 	int ret, len;
 	const char *cmd_args[2];
-	char cmd_output[MAXPATHLEN];
+	char cmd_output[PATH_MAX];
 
 	if (!specs)
 		return -1;
