@@ -51,6 +51,7 @@
 #include "lvm.h"
 #include "lxc.h"
 #include "lxclock.h"
+#include "memory_utils.h"
 #include "namespace.h"
 #include "nbd.h"
 #include "overlay.h"
@@ -355,7 +356,6 @@ struct lxc_storage *storage_copy(struct lxc_container *c, const char *cname,
 	}
 
 	if (!orig->dest) {
-		int ret;
 		size_t len;
 		struct stat sb;
 
@@ -568,13 +568,10 @@ struct lxc_storage *storage_create(const char *dest, const char *type,
 
 	/* -B lvm,dir */
 	if (strchr(type, ',')) {
-		char *dup, *token;
-		size_t len;
-
-		len = strlen(type);
-		dup = alloca(len + 1);
-		(void)strlcpy(dup, type, len + 1);
+		__do_free char *dup = NULL;
+		char *token;
 
+		dup = must_copy_string(type);
 		lxc_iterate_parts(token, dup, ",") {
 			bdev = do_storage_create(dest, token, cname, specs);
 			if (bdev)
